package server;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.codec.LengthFieldBasedFrameDecoder;
import io.netty.handler.codec.http.HttpObjectAggregator;
import io.netty.handler.codec.http.HttpServerCodec;
import io.netty.handler.codec.http.websocketx.WebSocketServerProtocolHandler;
import io.netty.handler.codec.string.StringDecoder;
import io.netty.handler.codec.string.StringEncoder;
import io.netty.handler.stream.ChunkedWriteHandler;
import io.netty.handler.timeout.IdleStateHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.InputStream;
import java.net.InetAddress;
import java.nio.ByteOrder;
import java.util.Properties;

/**
 * @author yuli
 * @Description:TODO
 * @ClassName TcpServer
 **/
public class TcpServer {

    private static Logger logger = LoggerFactory.getLogger(TcpServer.class);

    // 服务器地址端口
//    private static final String IP = "192.168.0.112";
//    private static final String IP = "192.168.0.107";
//    public static String IP = "8.143.203.103";
    private static final int PORT = 9095;



    /** 用于分配处理业务线程的线程组个数 */
    protected static final int BIZGROUPSIZE = Runtime.getRuntime().availableProcessors() * 2;
    /** 业务出现线程大小 */
    protected static final int BIZTHREADSIZE = 4;

    /*
     * NioEventLoopGroup实际上就是个线程池,
     * NioEventLoopGroup在后台启动了n个NioEventLoop来处理Channel事件,
     * 每一个NioEventLoop负责处理m个Channel,
     * NioEventLoopGroup从NioEventLoop数组里挨个取出NioEventLoop来处理Channel
     */
    private static final EventLoopGroup bossGroup = new NioEventLoopGroup(BIZGROUPSIZE);
    private static final EventLoopGroup workerGroup = new NioEventLoopGroup(BIZTHREADSIZE);

    //    线程内容
    protected static void run() throws Exception {
        ServerBootstrap b = new ServerBootstrap();
        b.group(bossGroup, workerGroup);
        b.channel(NioServerSocketChannel.class);
        b.childOption(ChannelOption.SO_REUSEADDR, true); //快速复用端口
        b.childOption(ChannelOption.CONNECT_TIMEOUT_MILLIS,1000);
        b.childHandler(new ChannelInitializer<SocketChannel>() {

            @Override
            public void initChannel(SocketChannel ch) throws Exception {
//                ChannelPipeline pipeline = ch.pipeline();
//                Decode是对发送的信息进行编码、
//                @param maxFrameLength  帧的最大长度
//                @param lengthFieldOffset length字段偏移的地址
//                @param lengthFieldLength length字段所占的字节
//                @param lengthAdjustment 修改帧数据长度字段中定义的值，
//                可以为负数 因为有时候我们习惯把头部记入长度,若为负数,则说明要推后多少个字段
//                @param initialBytesToStrip 解析时候跳过多少个长度
//                pipeline.addLast(new StringEncoder());
//                pipeline.addLast(new StringDecoder());
//                pipeline.addLast(new TcpServerHandler());
                //因为基于http协议，使用http的编码和解码器
//                pipeline.addLast(new HttpServerCodec());
//                //是以块方式写，添加ChunkedWriteHandler处理器
//                pipeline.addLast(new ChunkedWriteHandler());
//                // http数据聚合器 用于将大数据量分段传输的数据 聚合
//                pipeline.addLast(new HttpObjectAggregator(8192));
//                // websocket协议处理器
//                pipeline.addLast(new WebSocketServerProtocolHandler("/ws"));
//                // 自定义的业务处理
//                pipeline.addLast(new TcpServerHandler());


                ch.pipeline().addLast("active",new ChannelActiveHandler() );
                //Socket 连接心跳检测
                ch.pipeline().addLast("idleStateHandler", new IdleStateHandler(60, 0, 0));

                ch.pipeline().addLast("socketChoose",new SocketChooseHandler());

                //注意，这个专门针对 Socket 信息的解码器只能放在 SocketChooseHandler 之后，否则会导致 webSocket 连接出错
                ch.pipeline().addLast("myDecoder",new LengthFieldBasedFrameDecoder(ByteOrder.LITTLE_ENDIAN,1024*1024,0,4,0,4,true));
                ch.pipeline().addLast("commonhandler",new DeviceServerHandler());

            }
        });
//        异步绑定端口
        b.bind(PORT).sync();
        logger.info("TCP Server Started");
    }
//            关闭端口
    protected static void shutdown() {
        workerGroup.shutdownGracefully();
        bossGroup.shutdownGracefully();
    }

    public static void main(String[] args) throws Exception {
        logger.info("Starting TCP Server...");
        TcpServer.run();
        // TcpServer.shutdown();
    }
}
